/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package io;

import net.util.LinkExtractor;
import init.JID2;
import io.downloaders.newdownloader3.AppendToUrlAction;
import io.downloaders.newdownloader3.ConditionalRedirectAction;
import io.downloaders.newdownloader3.DisableGzipAction;
import io.downloaders.newdownloader3.RedirectAction;
import io.downloaders.newdownloader3.ReplaceInURLAction;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import plugin.PluginLoader;


import util.exception.CancelException;
import xml.reader.DefaultXMLReader;

import model.transfer.ConfigUpdate;
import model.urlconfig.URLConfig;
import model.urlconfig.URLContainsConfig;
import model.urlconfig.URLEndsWithConfig;
import model.urlconfig.URLMatchesConfig;
import model.urlconfig.URLNotContainsConfig;
import model.urlconfig.URLStartWithConfig;

public class ConfigLoader3 {
	private static Path cfgDir;
	private static PluginLoader pluginLoader = new PluginLoader();
//	private static final SimpleDateFormat DF=
//			new SimpleDateFormat("d MMM yyyy HH:mm:ss z",Locale.US);
	
	public static void setCfgDir(Path cfgDir) {
		ConfigLoader3.cfgDir = cfgDir;
	}
	
	public static IDownloaderConfig load(String host) throws IOException, CancelException {
		try {
			CfgLoaderHandler cl=new CfgLoaderHandler(host);
			Path f=cfgDir.resolve("hosts.xml");
			
			if (!Files.exists(f)) {
				downloadUpdates(true);
			}
			DefaultXMLReader.read(f.toFile(),cl );
			return cl.getCfg();
		} catch (SAXException e) {
			throw new IOException();
		}
	}

	private static URLConfig getUrlConfig(String method,String expr) {
		if (method.equals("matches")) {
			return new URLMatchesConfig(expr);
		} else if (method.equals("contains")) {
			return new URLContainsConfig(expr);
		} else if (method.equals("startsWith")) {
			return new URLStartWithConfig(expr);
		} else if (method.equals("endsWith")) {
			return new URLEndsWithConfig(expr);
		} else if (method.equals("notContains")) {
			return new URLNotContainsConfig(expr);
		} else {
			throw new IllegalArgumentException();
		}
	}


	public static String[] loadDelimiters() throws IOException {
		
		Path f=cfgDir.resolve("parser").resolve("delimiters.cfg");
		if (Files.exists(f)) {
			List<String>d= Files.readAllLines(f,JID2.UTF8);
			
			String[] delimiters=new String[d.size()];
		
			int i=0;
			for (String s:d) {
				
				if (s.equals("\\u0020")) {
					delimiters[i]="\u0020";
				} else if (s.equals("\\n")) {
					delimiters[i]=("\n");
				} else if (s.equals("\\r")) {
					delimiters[i]=("\r");
				} else { 
					delimiters[i]=s;
				}
				i++;
			}
			return delimiters;
		} else {
			throw new FileNotFoundException();
		}
		
	}

	private static class CfgLoaderHandler implements ContentHandler {
		
		private IDownloaderConfig cfg;
		private boolean found;
		private String host;
		
		public CfgLoaderHandler(String host) {
			this.host=host;
		}
		
		@Override
		public void setDocumentLocator(Locator locator) {
		}
		
		@Override
		public void startDocument() throws SAXException {
		}
		
		@Override
		public void endDocument() throws SAXException {
		}
		
		@Override
		public void startPrefixMapping(String prefix, String uri) throws SAXException {
		}
		
		@Override
		public void endPrefixMapping(String prefix) throws SAXException {
		}
		
		@Override
		public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
			if (localName.equals("host")){
				found = atts.getValue("name").equals(host);
				String a=atts.getValue("additional");
				boolean additional=a!=null&& a.equals("true");
				String strDirect=atts.getValue("direct");
				boolean direct=strDirect!=null&&strDirect.equals("true");
				if (found){
					if (!additional){
						cfg=new DefaultDownloaderConfig3();
					} else if (direct){
						cfg = new DirectDownloadCfg();
						cfg.addUrlConfig(new URLContainsConfig(atts.getValue("arg")));
					} else {
						Path f=JID2.getInstance().getConfigurationPath().resolve("additional").resolve(atts.getValue("module"));
						cfg=(IDownloaderConfig) pluginLoader.load(f, atts.getValue("class"));
						System.out.println(cfg);
					}
				}
					
					
			} else if (found) {
				if (localName.equals("url")) {
					if (atts.getLength() == 2) {
						System.out.println("add");
						URLConfig uc=getUrlConfig(
							atts.getValue("method"),
							atts.getValue("arg")
						);
						cfg.addUrlConfig(uc);
					}
				} else if (localName.equals("redirect")) {
					String type=atts.getValue("extractorType");
					if (type==null) throw new SAXException();
					LinkExtractor extractor = LinkExtractor.getLinkExtractor(type);
					
					String[] argNames = extractor.getArgNames();
					String[] extractorArgs = new String[argNames.length];
					
					for (int i=0;i<argNames.length;i++) {
						extractorArgs[i] = atts.getValue(argNames[i]);
					}
					
					RedirectAction a=new RedirectAction(extractor, extractorArgs);
					cfg.addAction(a);
				} else if (localName.equals("appendToUrl")) {
					cfg.addAction( new AppendToUrlAction(atts.getValue("arg") )); 
				} else if (localName.equals("conditionalRedirect")) {
					String type=atts.getValue("extractorType");
					if (type==null) throw new SAXException();
					LinkExtractor extractor = LinkExtractor.getLinkExtractor(type);
					
					String[] argNames = extractor.getArgNames();
					String[] extractorArgs = new String[argNames.length];
					
					for (int i=0;i<argNames.length;i++) {
						extractorArgs[i] = atts.getValue(argNames[i]);
					}
					
					String[] extractorArgsElse = new String[argNames.length];
					
					for (int i=0;i<argNames.length;i++) {
						extractorArgsElse[i] = atts.getValue(argNames[i]+"Else");
					}
					
					ConditionalRedirectAction a=new ConditionalRedirectAction(extractor, atts.getValue("contains"), extractorArgs, extractorArgsElse);
					cfg.addAction(a);
				} else if (localName.equals("replaceInUrl")) {
					cfg.addAction(new ReplaceInURLAction(atts.getValue("target"), atts.getValue("replace")));
				} else if (localName.equals("disableGzip")) {
					cfg.addAction(new DisableGzipAction());
				}
			}
		}
		
		@Override
		public void endElement(String uri, String localName, String qName) throws SAXException {
		}
		
		@Override
		public void characters(char[] ch, int start, int length) throws SAXException {
		}
		
		@Override
		public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
		}
		
		@Override
		public void processingInstruction(String target, String data) throws SAXException {
		}
		
		@Override
		public void skippedEntity(String name) throws SAXException {
		}
		
		public IDownloaderConfig getCfg() {
			return cfg;
		}
	}

	/*private static class ParserCfgLoaderHandler implements ContentHandler {

		private Map<String, IDownloaderConfig> map;
		
		public ParserCfgLoaderHandler(Map<String, IDownloaderConfig> map) {
			this.map=map;
		}
		
		@Override
		public void setDocumentLocator(Locator locator) {
			
			
		}

		@Override
		public void startDocument() throws SAXException {
			
			
		}

		@Override
		public void endDocument() throws SAXException {
			
			
		}

		@Override
		public void startPrefixMapping(String prefix, String uri) throws SAXException {
			
			
		}

		@Override
		public void endPrefixMapping(String prefix) throws SAXException {
			
			
		}

		@Override
		public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
			if (localName.equals("entry")) {
				DirectDownloadCfg cfg=new DirectDownloadCfg();
				cfg.addUrlConfig(new URLContainsConfig(atts.getValue("arg")));
				map.put(atts.getValue("host"), cfg);
			}
			
		}

		@Override
		public void endElement(String uri, String localName, String qName) throws SAXException {
			
			
		}

		@Override
		public void characters(char[] ch, int start, int length) throws SAXException {
			
			
		}

		@Override
		public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
			
			
		}

		@Override
		public void processingInstruction(String target, String data) throws SAXException {
			
			
		}

		@Override
		public void skippedEntity(String name) throws SAXException {
			
			
		}
		
	}
	
	public static Map<String, IDownloaderConfig> loadAdditionalConfigs() throws IOException {
		Map<String, IDownloaderConfig> map =new HashMap<>(100);
		
		ParserCfgLoaderHandler cl=new ParserCfgLoaderHandler(map);
		Path f=cfgDir.resolve("parser").resolve("parser.cfg");
		
		if (Files.exists(f)) {
			try {
				DefaultXMLReader.read(f.toFile(),cl );
			} catch (SAXException e) {
				throw new IOException(e);
			}
		}
		return map;
	}*/
	
	public static boolean downloadUpdates() throws IOException, CancelException{
		return downloadUpdates(false);
	}
	
	public static boolean downloadUpdates(boolean force) throws IOException, CancelException{
		String type = "additional";
		String name = "jid";
		Path additionalDir=cfgDir.resolve(type);
		
		boolean b1= ConfigUpdate.downloadUpdates(name, "hosts", JID2.getInstance(), cfgDir, force);
		boolean b2= ConfigUpdate.downloadNonXMLUpdates(name, type, JID2.getInstance(),additionalDir , force);
		
		return b1 || b2;		
	}
}
